{
***********************************************************
*     dlftool.pas - a program for manipulating DLF archives
*     Author: Prof A Olowofoyeku (The African Chief)
*     Licence : FREEWARE
*     Last amended: 26 August 2004
**************************************************************
}

PROGRAM dlftool;
{$I-,X+}

{$ifdef Win32}
 {$ifndef __GPC__}
   {$ifndef VirtualPascal}
     {$apptype console}
   {$endif}
 {$endif}
{$endif}

{.$define __DLL__}

USES
SysUtils,
{$ifdef __DLL__}
dlfdll_import;
{$else}
dlfunit;
{$endif}

// callback FOR listing files
FUNCTION ListCallBack ( H : pDLF_Header; F : pDLF_FileRecFull ) : Integer;
{$ifdef Win32}STDCALL;{$endif}
BEGIN
   Result := 0;
   IF ( Assigned ( h ) ) AND ( NOT Assigned ( f ) )
   THEN BEGIN
     Writeln ( 'List of contents of DLF file: ' );
     WITH h^
     DO BEGIN
       Writeln ( 'Number of files    = ', FileCount );
       Writeln ( 'Compression type   = ', compressiontype );
       Writeln ( 'Start of file data = byte offset ', StartOfFileData );
     END;
     Writeln;
     Writeln ( 'Archive details: ' );
  END ELSE
  IF Assigned ( f ) THEN
  WITH f^ DO BEGIN
       Writeln ( 'File number = ', FileID );
       Writeln ( 'File Name   = ', FName );
       Writeln ( 'File Size   = ', FSize, ' byte(s)' );
       Writeln ( 'Destination = ', FDir );
       Writeln ( 'StartOffset = byte ', Start );
       Writeln ( 'EndOffset   = byte ', Stop );
       Writeln;
  END ELSE Result := - 1;
END;

// callback FOR creating archive
FUNCTION CreateCallBack ( FileName : pChar; h : pDLF_HEADER;
ID, Size, Copied : Longint )
: Integer; {$ifdef Win32}STDCALL;{$endif}
BEGIN
   Result := 0;
   IF Copied = - 1
   THEN BEGIN
     Writeln ( 'Creating archive: ', StrPas ( FileName ) );
     Writeln ( 'Archive should contain ', h^.FileCount, ' file(s)' );
   END ELSE
   IF Copied = - 100 THEN Writeln ( 'Adding file #', ID, ': ', StrPas ( FileName ), '...' )
END;

// callback FOR extracting files from archive
FUNCTION ExtractCallBack ( FileName : pChar; h : pDLF_HEADER;
ID, Size, Copied : Longint )
: Integer; {$ifdef Win32}STDCALL;{$endif}
BEGIN
   Result := 0;
   IF Copied = - 1
   THEN BEGIN
     Writeln ( 'Extracting files from archive: ', StrPas ( FileName ) );
     Writeln ( 'Archive contains ', h^.FileCount, ' file(s)' );
   END
   ELSE
   IF Copied = - 100 THEN Writeln ( 'Extracting file #', ID, ': ', StrPas ( FileName ), '...' )
END;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
VAR
s, s2 : AnsiString;
BEGIN
   WRITE ( DLFTitle, DLFVersion );
   Writeln ( ',', DLFAuthor );
   IF ( ParamCount = 0 )
   THEN BEGIN
      s := ExtractFileName ( ParamStr ( 0 ) );
      s := ChangeFileExt ( s, '' );

      Writeln ( 'Syntax = ', s, ' <DLF_file> [[-v][-d=<target directory>][-f=filespecs]]' );
      Writeln ( '         "-v"  : list contents of DLF file ' );
      Writeln ( '                 e.g., ', s, ' test.dlf -v' );
      Writeln ( '         "-d=" : extract files to specified directory' );
      {$ifdef linux}
      Writeln ( '                 e.g., ', s, ' test.dlf -d=/tmp/dlf' );
      Writeln ( '         "-a=" : archive the specified file specs into DLF' );
      Writeln ( '                 e.g., ', s, ' test.dlf -a=/tmp/dlf/*' );
     {$else}
      Writeln ( '                 e.g., ', s, ' test.dlf -d=c:\temp\dlf' );
      Writeln ( '         "-a=" : archive the specified file specs into DLF' );
      Writeln ( '                 e.g., ', s, ' test.dlf -a=c:\temp\dlf\*.*' );
      {$endif}
      Halt;
   END;

   s2 := ParamStr ( 1 );
   IF ( ParamCount = 1 ) THEN ExtractDLF ( pChar ( s2 ), '', ExtractCallBack )
   ELSE BEGIN
      s := ParamStr ( 2 );
      IF ( UpperCase ( s ) = '-V' ) OR ( UpperCase ( s ) = '/V' )
         THEN ListDLF ( pChar ( s2 ), ListCallBack )
      ELSE
      IF Pos ( '-A=', UpperCase ( s ) ) > 0
      THEN BEGIN
          Delete ( s, 1, 3 );
          s := Trim ( s );
          Writeln ( CreateDLF ( pChar ( s2 ), pChar ( s ), CreateCallBack ), ' file(s) archived.' );
      END
      ELSE BEGIN
           IF Pos ( '-D=', UpperCase ( s ) ) > 0
           THEN BEGIN
              Delete ( s, 1, 3 );
              s := Trim ( s );
           END ELSE BEGIN
              Writeln ( 'Unrecognised switch "', s, '" - extracting to current directory.' );
              s := '';
           END;
           Writeln ( ExtractDLF ( pChar ( s2 ), pChar ( s ), ExtractCallBack ), ' file(s) extracted.' );
       END;
   END;
END.

