{*
 ***********************************************************************
 *
 *   Recursive File Search library for Dos, Windows, Linux, etc.
 *   (c) 2000-2004, Prof. A Olowofoyeku (The African Chief)
 *             <African_Chief@bigfoot.com>
 *   Supports
 *    GNU Pascal, Borland Pascal, Delphi, Virtual Pascal
 *
 *   This library is FREEWARE (not Public Domain)
 *
 ***********************************************************************
 *}
UNIT searchr;

INTERFACE

{$i searchr.inc}

USES
{$ifdef ver70}
  {$ifdef MSWindows}
  WinDos,
  {$else}
  Dos,
  {$endif}
  Strings
{$else}  {ver70}
  {$ifndef UsingSysUtils}
  Dos,
  Strings
  {$else UsingSysUtils}
  SysUtils
  {$endif UsingSysUtils}
{$endif} {ver70}
  {$ifdef __GPC__}
  , gpc
  {$endif};


{$ifndef UsingSysUtils}
 {$ifndef MSwindows}
  CONST
  faanyfile   = anyfile;
  fadirectory = directory;
  fareadonly  = readonly;
  faarchive   = archive;
  fahidden    = hidden;
  favolumeid  = volumeid;
  fasysfile   = sysfile;

  TYPE
  TSearchRec = SearchRec;
 {$endif}
{$endif}

{ numerical code to indicate that the user stopped the search }
CONST
UserAbortCode = - 1000;

{ maximum size for basic string types }
CONST
Max_Str_Size         =  {$ifdef __OS_DOS__}255{$else}2048{$endif};

{ separators for directories }
CONST
NativeOSDirSeparator =  {$ifdef __POSIX__}'/'{$else}'\'{$endif};
OtherOSDirSeparator  =  {$ifdef __POSIX__}'\'{$else}'/'{$endif};
GlobalWildCard       =  {$ifdef __POSIX__}'*'{$else}'*.*'{$endif};
Unix_OS              =  {$ifdef __POSIX__}TRUE{$else}FALSE{$endif};

TYPE
TSearchString = String {$ifndef Kylix} [Max_Str_Size]{$endif};
TSearchFunc   = FUNCTION ( CONST fName : String; VAR Head :  TSearchRec ) : Longint;

{ exported routines }
PROCEDURE RecursiveSearch ( CONST StartDir, FileSpec : String; aFunc : TSearchFunc );
FUNCTION  MatchFileName ( CONST FileSpec, StringToSearch : String ) : Boolean;
FUNCTION  CompareFileNames ( CONST s1, s2 : String ) : integer;
FUNCTION  CurrentRootDirectory : TSearchString;

{$ifndef UsingSysUtils}
FUNCTION InttoStr ( i : integer ) : String;
FUNCTION UpperCase ( s : string ) : String;
FUNCTION AnsiCompareText ( s1, s2 : String ) : integer;
FUNCTION ExpandFileName ( s : string ) : String;
FUNCTION ExtractFileName ( s : string ) : String;
FUNCTION ExtractFileDir ( CONST s : string ) : String;
FUNCTION ExtractFilePath ( CONST s : string ) : String;
{$endif}

IMPLEMENTATION

PROCEDURE RecursiveSearch ( CONST StartDir, FileSpec : String; aFunc : TSearchFunc );
VAR
  ThisDir : TSearchString;
  SearchRec : TSearchRec;
  Found : Integer;
BEGIN
  ThisDir := StartDir;

  { remove trailing slash }
  WHILE ThisDir [Length ( ThisDir ) ] IN [NativeOSDirSeparator, OtherOSDirSeparator]
    DO ThisDir := Copy ( ThisDir, 1, Length ( ThisDir ) - 1 );
  {
  WHILE Copy ( ThisDir, Length ( ThisDir ), 1 ) = NativeOSDirSeparator
    DO ThisDir := Copy ( ThisDir, 1, Length ( ThisDir ) - 1 );
  }

  { start looking }
  {$ifdef UsingSysUtils} Found := {$endif}
  FindFirst ( ThisDir + NativeOSDirSeparator + GlobalWildCard, faAnyfile, SearchRec );
  {$ifndef UsingSysUtils}Found := DosError;{$endif}
  WHILE Found = 0
  DO BEGIN
    IF ( ( SearchRec.Attr AND faDirectory ) > 0 )
    THEN BEGIN
      IF ( ( SearchRec.Name ) <> '.' ) AND ( ( SearchRec.Name ) <> '..' )
      THEN BEGIN
        IF Assigned ( aFunc )
        THEN BEGIN
          IF aFunc ( ThisDir + NativeOSDirSeparator + ( SearchRec.Name ), SearchRec ) <> 0
          THEN BEGIN
             Found := UserAbortCode;
             Break;
          END;
        END;
        RecursiveSearch ( ThisDir + NativeOSDirSeparator + ( SearchRec.Name ), FileSpec, aFunc );
      END;
    END
    ELSE BEGIN
      IF Assigned ( aFunc )
      THEN BEGIN
          IF aFunc ( ThisDir + NativeOSDirSeparator + ( SearchRec.Name ), SearchRec ) <> 0
          THEN BEGIN
             Found := UserAbortCode;
             Break;
          END;
      END;
    END;
    {$ifdef UsingSysUtils}  Found := {$endif}FindNext ( SearchRec );
    {$ifndef UsingSysUtils} Found := DosError;{$endif}
  END; { while found = 0 }
  {$ifdef UsingSysUtils}FindClose ( SearchRec );{$endif}
END;


FUNCTION MatchFileName ( CONST FileSpec, StringToSearch : String ) : Boolean;
{ does filename: StringtoSearch match the file specification: FileSpec? }
VAR
pStringToSearch,
pFileSpec : pChar;
b1 : boolean;

FUNCTION MatchPattern ( Element, FileSpec : PChar ) : Boolean;

FUNCTION IsPatternWild ( FileSpec : PChar ) : Boolean;
VAR
  b : Boolean;
BEGIN
  b := StrScan ( FileSpec, '*' ) <> NIL;
  IF NOT b THEN b := StrScan ( FileSpec, '?' ) <> NIL;
  IsPatternWild := b;
END;

BEGIN
  b1 := False;
  IF 0 = StrComp ( FileSpec, '*' ) THEN
    b1 := True
  ELSE IF ( element^ = Chr ( 0 ) ) AND ( FileSpec^ <> Chr ( 0 ) ) THEN
    b1 := False
  ELSE IF element^ = Chr ( 0 ) THEN
    b1 := True
  ELSE BEGIN
    CASE FileSpec^ OF
    '*' : IF MatchPattern ( element, @FileSpec [1] ) THEN
           b1 := True
         ELSE
           b1 := MatchPattern ( @element [1], FileSpec );
    '?' : b1 := MatchPattern ( @element [1], @FileSpec [1] );

    ELSE
      IF element^ = FileSpec^ THEN
        b1 := MatchPattern ( @element [1], @FileSpec [1] )
      ELSE
        b1 := False;
    END;  { case }
  END; { else begin }
  MatchPattern := b1;
END; { MatchPattern }

CONST StrMaxLen = 255;

BEGIN{ MatchFileName }
  MatchFileName := TRUE;
  IF ( FileSpec = '*.*' ) OR ( FileSpec = '' ) OR ( FileSpec = '*' )
  THEN exit; { '' or '*' or '*.*' = all files match}

  Getmem ( pStringToSearch, StrMaxLen );
  Getmem ( pFileSpec, StrMaxLen );
  StrPCopy ( pStringToSearch, StringToSearch );
  StrPCopy ( pFileSpec, FileSpec );
  {$ifdef __OS_DOS__}  { dos, windows, os/2, djgpp - case-insensitive search }
                       { linux, unix - case-sensitive search }
  StrUpper ( pStringToSearch );
  StrUpper ( pFileSpec );
  {$endif}
  MatchFileName := MatchPattern ( pStringToSearch, pFileSpec );
  Freemem ( pFileSpec, StrMaxLen );
  Freemem ( pStringToSearch, StrMaxLen );
END; {* MatchFileName *}

FUNCTION CompareFileNames ( CONST s1, s2 : String ) : integer;
BEGIN
  {$ifdef __OS_DOS__}
  CompareFileNames := AnsiCompareText ( s1, s2 );
  {$else}
  {$ifdef Kylix}
    CompareFileNames := AnsiCompareStr ( s1, s2 );
  {$else}
    CompareFileNames := StrComp ( s1, s2 );
  {$endif}
  {$endif}
END; {* CompareFileNames *}


FUNCTION  CurrentRootDirectory : TSearchString;
BEGIN
  CurrentRootDirectory :=
  {$ifdef UsingSysUtils}
   ExtractFileDrive ( GetCurrentDir ) + NativeOSDirSeparator;
  {$else}
   NativeOSDirSeparator;
  {$endif}
END; {* CurrentRootDirectory *}

{$ifndef UsingSysUtils}
FUNCTION UpperCase ( s : string ) : String;
VAR
i : Word;
BEGIN
  FOR i := 1 TO length ( s ) DO s [i] := Upcase ( s [i] );
  UpperCase := s;
END;

FUNCTION AnsiCompareText ( s1, s2 : String ) : integer;
BEGIN
  IF UpperCase ( s1 ) = UpperCase ( s2 ) THEN AnsiCompareText := 0
  ELSE BEGIN
     s1 := s1 + #0;
     s2 := s2 + #0;
     AnsiCompareText := StriComp ( @s1 [1], @s2 [1] );
  END;
END;

FUNCTION ExpandFileName ( s : string ) : String;
VAR
p : pChar;
BEGIN
   ExpandFileName := s;
   {$ifdef MSwindows}
   Getmem ( p, 255 );
   strpcopy ( p, s );
   FileExpand ( p, p );
   IF Strlen ( p ) > 0 THEN ExpandFileName := Strpas ( p );
   Freemem ( p, 255 );
   {$else}
   s := FExpand ( s );
   IF s <> '' THEN ExpandFileName := s;
  {$endif}
END;

FUNCTION ExtractFileName ( s : string ) : String;
VAR
i : Word;
BEGIN
  ExtractFileName := s;
  FOR i := length ( s ) DOWNTO 1
  DO BEGIN
      IF s [i] IN [':', '/', '\']
      THEN BEGIN
         Delete ( s, 1, i );
         ExtractFileName := s;
         Exit;
      END;
  END;
END;

FUNCTION ExtractFileDir ( CONST s : string ) : String;
VAR
i : Word;
BEGIN
  ExtractFileDir := '';
  FOR i := length ( s ) DOWNTO 1
  DO BEGIN
      IF s [i] IN [':', '/', '\']
      THEN BEGIN
         IF s [i] = ':' THEN inc ( i );
         ExtractFileDir := Copy ( s, 1, Pred ( i ) );
         Exit;
      END;
  END;
END;

FUNCTION ExtractFilePath ( CONST s : string ) : String;
VAR
i : Word;
BEGIN
  ExtractFilePath := '';
  FOR i := length ( s ) DOWNTO 1
  DO BEGIN
      IF s [i] IN [':', '/', '\']
      THEN BEGIN
         ExtractFilePath := Copy ( s, 1, i );
         Exit;
      END;
  END;
END;

FUNCTION InttoStr ( i : integer ) : String;
VAR
s : String [80];
BEGIN
  Str ( i, s );
  InttoStr := s;
END;
{$endif}

{ () }
BEGIN
END.

